import network
import json
from emp_utils import rainbow
from emp_utils import print_as_a_list_item
from emp_utils import selection
from emp_utils import config_path


class Wifi:
    _profile = 'config/emp_wifi.json'

    @classmethod
    def create_profile(cls):
        default = ()
        records = []
        default_config = dict(default=default, records=records)
        cls.update_profile(default_config)
        print(rainbow('创建 config/wifi_cfg.json 配置', color='blue'))

    @classmethod
    def update_profile(cls, config):
        config_path()
        with open(cls._profile, 'w') as f:
            f.write(json.dumps(config))

    @classmethod
    def read_config(cls):
        try:
            with open(cls._profile, 'r') as f:
                return json.loads(f.read())
        except:
            cls.create_profile()
            default = ()
            records = []
            return dict(default=default, records=records)

    @classmethod
    def get_default(cls):
        config = cls.read_config()
        return config.get('default') if config else ()

    @classmethod
    def set_default(cls, essid):
        config = cls.read_config()
        records = config.get('records')

        for index, item in enumerate(records):
            print(print_as_a_list_item(index, item))

        default = selection(
            '请选择一个选项作为默认wifi连接 [0-%s]' % str(
                len(records - 1)), len(records - 1))

        config['default'] = records[default]
        cls.update_profile(config)

    @classmethod
    def get_records(cls):
        config = cls.read_config()
        return config.get('records') if config else []

    @classmethod
    def is_in_records(cls, essid):
        for i in cls.get_records():
            if essid == i[0]:
                return i
        return ()

    @classmethod
    def add_record(cls, essid, passwd):
        config = cls.read_config()
        config['records'].append((essid, passwd))
        cls.update_profile(config)
        print(rainbow('添加记录: %s' % essid, color='green'))

    @classmethod
    def del_record(cls, essid):
        config = cls.read_config()
        for index, i in enumerate(config.get('records')):
            if essid == i[0]:
                config['records'].pop(index)
                cls.update_profile(config)
                print(rainbow('记录删除: %s' % essid, color='green'))
                return True
        return False

    @classmethod
    def update_record(cls, essid, passwd):
        config = cls.read_config()
        for index, i in enumerate(config['records']):
            if i[0] == essid:
                config['records'][index] = (essid, passwd)
                return True
        return False

    @classmethod
    def ifconfig(cls):
        return NetWorker.worker().ifconfig()

    @classmethod
    def connect(cls):
        worker = NetWorker.worker()
        if worker.is_connected():
            s0 = '您已成功建立了WiFi连接.'
            print(rainbow(s0, color='green'))
        else:
            default = cls.get_default()
            if default:
                records = cls.get_records().insert(0, default)
            else:
                records = cls.get_records()
            # print(records)
            networks = [i.get('essid') for i in worker.scan()]
            for i in records:
                if i[0] in networks:
                    s1 = '尝试自动连接到 %s ...' % i[0]
                    print(rainbow(s1, color='blue'))
                    if not worker.do_connect(*i):
                        s2 = '自动连接 %s failed' % i[0]
                        print(rainbow(s2, color='red'))
                        cls.del_record(i[0])
                        worker._wifi.active(True)
                        continue
                    else:
                        s3 = '自动连接到 %s 成功' % i[0]
                        print(rainbow(s3, color='green'))
                        return True
            print(rainbow('没有记录!', color='red'))
            worker.before_connect()

    @classmethod
    def disconnect(cls):
        NetWorker.worker().disconnect()


class NetWorker:
    _instance = None

    def __new__(cls):
        if not cls._instance:
            cls._instance = super(NetWorker, cls).__new__(cls)
            cls._instance._wifi = network.WLAN(network.STA_IF)
            cls._instance._wifi.active(True)
            cls._instance._essid = None
        return cls._instance

    def scan(self):
        def _list_wifi(index, essid, dbm):
            _index = ('[%s]' % str(index)).center(8).lstrip()
            _essid = rainbow(essid + (40 - len(essid)) * ' ', color='red')
            _dbm = rainbow(dbm.center(10).lstrip(), color='blue')
            print('{0} {1} {2} dBm'.format(_index, _essid, _dbm))

        # TODO 字符编码容错

        networks = []
        for i in self._wifi.scan():
            try:
                nw = dict(essid=i[0].decode(), dbm=str(i[3]))
            except:
                nw = dict(essid=i[0], dbm=str(i[3]))
            finally:
                networks.append(nw)
        # networks = [dict(essid=i[0].decode(),dbm=str(i[3])) for i in self._wifi.scan()]

        for i, item in enumerate(networks):
            _list_wifi(i, item['essid'], item['dbm'])

        return networks

    def ifconfig(self):
        info = self._wifi.ifconfig()
        print(rainbow('你连接的WiFi为 %s' % self._essid))
        print(rainbow('IP地址: ' + info[0], color='red'))
        print(rainbow('子网掩码: ' + info[1], color='green'))
        print(rainbow('网关: ' + info[2], color='blue'))
        return info, self._essid

    def is_connected(self):
        return self._wifi.isconnected()

    def before_connect(self):
        print('正在扫描网络...')
        networks = self.scan()
        selection = input(
            '您想访问哪一个? [0-%s]' % str(len(networks) - 1))
        if int(selection) > len(networks) - 1:
            print('你的选择超出了范围!')
            self.before_connect()
        essid = networks[int(selection)]['essid']

        passwd = input('请输入WiFi密码 %s: ' % essid)

        if not self.do_connect(essid, passwd):
            self.before_connect()

    def do_connect(self, essid, passwd):
        if not self._wifi.isconnected():
            print('正在连接到网络...')
            self._wifi.active(True)
            self._wifi.connect(essid, passwd)
            import utime

            for i in range(300):
                if self._wifi.isconnected():
                    break
                utime.sleep_ms(100)

            if not self._wifi.isconnected():
                self._wifi.active(False)
                print(
                    rainbow(
                        'WiFi连接错误，请重新连接',
                        color='red'))
                return False

            else:
                self._essid = essid
                self.ifconfig()
                if not Wifi.is_in_records(essid):
                    Wifi.add_record(essid, passwd)

                return True
        else:
            print(rainbow('您已经建立了WiFi连接.', color='green'))
            return True

    def disconnect(self):
        self._wifi.active(False)
        print(rainbow('WIFI 连接已断开', color='red'))

    @classmethod
    def worker(cls):
        return NetWorker()


if __name__ == '__main__':
    Wifi.connect()



import gc
import json
import socket

import _webrepl
import network
import uos
import websocket_helper
from emp_utils import rainbow

try:
    import websocket
except ImportError:
    import uwebsocket as websocket


class WebREPL:
    _instance = None

    @classmethod
    def send(cls, json_data):
        WebREPL().ws.write(json_data)

    def __new__(cls):
        if not cls._instance:
            cls._instance = super(WebREPL, cls).__new__(cls)
            cls._instance.ws = None
            cls._instance.listen_s = None
            cls._instance.client_s = None
            cls._instance.wr = None
        return cls._instance

    @classmethod
    def setup_conn(cls, port, accept_handler):
        WebREPL().listen_s = socket.socket()
        WebREPL().listen_s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        ai = socket.getaddrinfo("0.0.0.0", port)
        addr = ai[0][4]

        WebREPL().listen_s.bind(addr)
        WebREPL().listen_s.listen(1)
        if accept_handler:
            WebREPL().listen_s.setsockopt(socket.SOL_SOCKET, 20, accept_handler)
        for i in (network.AP_IF, network.STA_IF):
            iface = network.WLAN(i)
            if iface.active():
                print(rainbow("WebREPL 启动守护进程 ws://%s:%d" %
                              (iface.ifconfig()[0], port), color='green'))
        return WebREPL().listen_s

    @classmethod
    def accept_conn(cls, listen_sock):

        cl, remote_addr = listen_sock.accept()
        prev = uos.dupterm(None)
        uos.dupterm(prev)
        if prev:
            print("\n多个 WebREPL 连接",
                  remote_addr, "已拒绝！")
            cl.close()
            return
        print("\nWebREPL 连接:", remote_addr)
        WebREPL().client_s = cl
        websocket_helper.server_handshake(cl)
        WebREPL().ws = websocket.websocket(cl, True)

        WebREPL().wr = _webrepl._webrepl(WebREPL().ws)
        type(WebREPL().wr)
        cl.setblocking(False)
        # notify REPL on socket incoming data
        cl.setsockopt(socket.SOL_SOCKET, 20, uos.dupterm_notify)
        uos.dupterm(WebREPL().wr)

    @classmethod
    def stop(cls):
        uos.dupterm(None)
        if WebREPL().client_s:
            WebREPL().client_s.close()
        if WebREPL().listen_s:
            WebREPL().listen_s.close()

    @classmethod
    def start(cls, port=8266, password=None):
        WebREPL().stop()
        if password is None:
            try:
                import webrepl_cfg
                _webrepl.password(webrepl_cfg.PASS)
                WebREPL().setup_conn(port, WebREPL().accept_conn)
                print("WebREPL 将按配置启用")
            except:
                print("WebREPL 未初始化配置, 请运行 'import webrepl_setup'")
        else:
            _webrepl.password(password)
            WebREPL().setup_conn(port, WebREPL().accept_conn)
            print(rainbow("WebREPL 启动.", color='green'))

    @classmethod
    def start_foreground(cls, port=8266):
        WebREPL().stop()
        s = WebREPL().setup_conn(port, None)
        WebREPL().accept_conn(s)


def emp_sender(func):
    def wrapper(*args, **kwargs):
        rsp = dict(func=func.__name__, data=func(*args, **kwargs))
        WebREPL.send(json.dumps(rsp) + '\n\r')
        gc.collect()
    return wrapper



import gc
import os
import sys

from emp_utils import is_folder
from emp_utils import traverse
from emp_webrepl import emp_sender


@emp_sender
def device_info():
    return dict(
        platform=sys.platform,
        version=sys.version,
        implementation=[sys.implementation[0], list(sys.implementation[1])],
        maxsize=sys.maxsize / 1024 / 1024)


@emp_sender
def memory_status():
    gc.collect()
    return dict(alloced=gc.mem_alloc() / 1024, free=gc.mem_free() / 1024)


@emp_sender
def memory_analysing(filename):
    gc.collect()
    fsize = os.stat(filename)[6]
    mf = gc.mem_free()
    return dict(fsize=fsize, mf=mf, filename=filename)


@emp_sender
def tree(path='/'):
    return traverse(path)


@emp_sender
def get_code(filename):
    gc.collect()
    with open(filename, 'r') as f:
        code = f.read()
        rsp = dict(code=code, filename=filename)
        memory_status()
        return rsp


def new_folder(folder):
    try:
        os.mkdir(folder)
    except:
        pass
    tree()


def new_file(filename):
    with open(filename, 'w') as f:
        print(f.write(''))
    tree()


def del_folder(path):
    for i in os.listdir(path):
        if is_folder(path + '/' + i):
            del_folder(path + '/' + i)
        else:
            os.remove(path + '/' + i)

    os.rmdir(path)
    tree()


def del_file(filename):
    os.remove(filename)
    tree()


def rename(old_name, new_name):
    try:
        os.rename(old_name, new_name)
        tree()
    except:
        pass


def emp_install(pkg):
    try:
        upip.install(pkg)
    except:
        import upip
        upip.install(pkg)




from emp_utils import rainbow
from emp_utils import print_as_a_list_item
from emp_utils import selection
from emp_utils import _const
from emp_utils import config_path
import os
import machine

BOOT_MODE = _const()
BOOT_MODE.WITH_NOTHING = 0
BOOT_MODE.WITH_WIFI_STARTUP = 1
BOOT_MODE.EASY_DEVELOP = 2

BOOT_MODE.WITH_WIFI_STARTUP_CODE = '''from emp_wifi import Wifi

if __name__ == '__main__':
    Wifi.connect()'''

BOOT_MODE.EASY_DEVELOP_CODE = '''from emp_wifi import Wifi
from emp_webrepl import WebREPL
from emp_utils import webrepl_pass
from emp_utils import post_ip

if __name__ == '__main__':
    print()
    print('       ---------------------------')
    print('       - Python YI MicroPython   -')
    print('       -      version 1.0.5      -')
    print('       -     by YI               -')
    print('       ---------------------------')
    print()
    Wifi.connect()
    try:
        post_ip(Wifi.ifconfig()[0][0])
    except ImportError:
        pass
    WebREPL.start(password=webrepl_pass())
    import ide'''


def reboot():
    print(rainbow('Reboot', color='red'))
    machine.reset()


def set_boot_mode():
    print(
        print_as_a_list_item(
            0, '清除模式',
            '注意: 清除引导程序 boot.py, 这将导致程序无法启动!'))
    print(
        print_as_a_list_item(
            1, 'WiFi模式',
            '此选项适合稳定程序使用！'))
    print(
        print_as_a_list_item(
            2, '开发者模式',
            '启动连接WIFI热点，并开启WebREPL开发模式'
        ))

    mode = selection('请选择模式 [0-2]: ', 2)

    with open('boot.py', 'w') as f:
        if mode == BOOT_MODE.WITH_NOTHING:
            boot_code = ''
            f.write(boot_code)
            print(rainbow('已设置为清除模式', color='green'))
        elif mode == BOOT_MODE.WITH_WIFI_STARTUP:
            boot_code = BOOT_MODE.WITH_WIFI_STARTUP_CODE
            f.write(boot_code)
            print(rainbow('已设置为WiFi模式', color='green'))

        elif mode == BOOT_MODE.EASY_DEVELOP:
            config_path()
            if not 'webrepl.pass' in os.listdir('config'):
                with open('config/webrepl.pass', 'w') as c:
                    c.write('123456')
            boot_code = BOOT_MODE.EASY_DEVELOP_CODE
            f.write(boot_code)
            print(rainbow('已设置为开发者模式', color='green'))

    reboot()


def set_web_repl():
    pw = input(rainbow('请输入新的WebREPL密码：', color='blue'))
    config_path()
    with open('config/webrepl.pass', 'w') as c:
        c.write(pw)
    print(rainbow('已重新设置WebREPL密码，重启之后可用！！！', color='green'))



import gc
import os


class _const:
    class ConstError(TypeError):
        pass

    def __setattr__(self, name, value):
        if self.__dict__.get(name):
            raise self.ConstError("Can't rebind const (%s)" % name)
        else:
            self.__dict__[name] = value


def is_folder(path):
    try:
        os.listdir(path)
        return True
    except:
        return False


def post_ip(ip):
    import urequests
    urequests.post('http://www.1zlab.com/ide/post/ip/?esp_ip=%s,' % ip)


def traverse(path):
    n = dict(name=path, children=[])
    for i in os.listdir(path):
        if is_folder(path + '/' + i):
            n['children'].append(traverse(path + '/' + i))
        else:
            n['children'].append(dict(name=path + '/' + i))
    return n


def config_path():
    try:
        return len(os.listdir('config'))
    except:
        os.mkdir('config')
    finally:
        return len(os.listdir('config'))


def webrepl_pass():
    with open('config/webrepl.pass', 'r') as f:
        return f.read()


def rainbow(output, color=None):
    if color:
        if color == 'green':
            return '\033[1;32m%s\033[0m' % output
        if color == 'red':
            return '\033[1;31m%s\033[0m' % output
        if color == 'blue':
            return '\033[1;34m%s\033[0m' % output
    else:
        return output


def print_left_just(output, length=None):
    if length == None:
        length = len(output)
    return output + (length - len(output)) * ' '


def print_right_just(output, length):
    if length == None:
        length = len(output)
    return (length - len(output)) * ' ' + output


def print_as_a_list_item(index, title, subtile=None):
    index = ('[%s]' % str(index)).center(8).lstrip()
    title = print_left_just(rainbow(title, color='green'))
    if subtile:
        subtile = '\n' + len(index) * ' ' + subtile
    else:
        subtile = ''
    return index + title + subtile


def selection(hint, range):
    index = input(rainbow(hint, color='blue'))
    if int(index) > range or int(index) < 0:
        print(rainbow('out of range!', color='red'))
        selection(hint, range)
    else:
        return int(index)


def mem_analyze(func):
    """
    装饰器:内存分析
    """

    def wrapper(*args, **kwargs):
        memory_alloc = '总内存: %s kb' % str(gc.mem_alloc() / 1024)
        memory_free = '可用内存: %s kb' % str(gc.mem_free() / 1024)
        gc.collect()
        memory_after_collect = '获得: %s kb 可用内存' % str(
            gc.mem_free() / 1024)
        print(rainbow(memory_alloc, color='red'))
        print(rainbow(memory_free, color='green'))
        print(rainbow(memory_after_collect, color='blue'))
        func(*args, **kwargs)
        memory_after_func_excute = '清除 %s 后: %s kb 可用内存' % (
            func.__name__, str(gc.mem_free() / 1024))
        print(rainbow(memory_after_func_excute, color='red'))

    return wrapper


def sync_time():
    import urequests
    from machine import RTC
    rtc = RTC()
    print('before sync: ', rtc.datetime())
    time = urequests.get('http://www.1zlab.com/api/get-time/').json()
    # print(time)
    rtc.init(tuple(time['rtc']))
    print('after sync: ', rtc.datetime())



from distutils.core import setup

setup(
    name='yi-mp',
    version='1.0.5',
    py_modules=['emp_wifi', 'emp_boot', 'ide', 'emp_utils', 'emp_webrepl'],
    author='1045149087@qq.com',
    author_email='1045149087@qq.com',
    url='https://github.com/HWYWL/yi-mp',
    description='(YI MicroPython) is a upy module to make things Easy on MicroPython.'
)
